-- prepared by Steve Stedman
-- http://SteveStedman.com
-- twitter:  @sqlEmt

RAISERROR ('Dont run the whole script, just run sections at a time', 20, 1)  WITH LOG;

------------------------ EXAMPLES SETUP ------------------------

USE Master;
GO

IF EXISTS(SELECT name 
            FROM sys.databases 
		   WHERE name = 'QueryTraining')
BEGIN
	ALTER DATABASE [QueryTraining] 
	  SET SINGLE_USER WITH ROLLBACK IMMEDIATE;
	DROP DATABASE [QueryTraining];
END
CREATE DATABASE [QueryTraining];
GO

USE [QueryTraining];
GO

-- wondering why the identity is seeded with the large negative value
-- http://stevestedman.com/?p=2098
CREATE TABLE [dbo].[Inventory](
	[InventoryID] [INTEGER] IDENTITY(-2147483647,1) PRIMARY KEY,
	[Category] [VARCHAR](200), 
	[ItemSKU] [VARCHAR](200),
	[Description] [VARCHAR](4000),
	[QuantityOnHand] [INT],
	[UnitCost] DECIMAL(10,2),
	[UnitRetail] DECIMAL(10,2),
	[Archived] BIT
); 


INSERT INTO dbo.Inventory (Category, ItemSKU, Description, QuantityOnHand,
                           UnitCost, UnitRetail, Archived)
VALUES
('SLEEPINGBAG', 'SB001-LG-RED', 'Sleeping Bag - Large - RED', 5, 5.75, 19.99, 0),
('SLEEPINGBAG', 'SB001-LG-GRN', 'Sleeping Bag - Large - GREEN', 13, 5.75, 14.99, 1),
('SLEEPINGBAG', 'SB001-LG-BLK', 'Sleeping Bag - Large - BLACK', 1, 5.75, 19.99, 0),
('TENT', 'TT001-10-CMO', 'Tent - 10 Man - CAMO', 5, 10.00, 99.95, 0),
('AXE', 'AX013-HT-BLK', 'Axe - Hatchet - Black', 35, 4.99, 24.95, 0),
('FOOD', 'MR005-1S-SPG', 'MRE - 1 Serve - Spaghetti', 5, 0.50, 7.99, 0),
('FOOD', 'MR006-1A-PIZ', 'Pizza', 1, NULL, 9.95, 0);



CREATE TABLE [Departments] (
 id INT, --would normally be an INT IDENTITY
 department VARCHAR (200),
 parent INT
 );

-- insert top level departments
INSERT INTO [Departments] (id, department, parent)
VALUES (1, 'Camping', NULL),
 (2, 'Cycle', NULL),
 (3, 'Snowsports', NULL),
 (4, 'Fitness', NULL);
 
-- now some sub-departments for camping
INSERT INTO [Departments] (id, department, parent)
VALUES (5, 'Tents', 1),
 (6, 'Backpacks', 1),
 (7, 'Sleeping Bags', 1),
 (8, 'Cooking', 1);
 
GO
SELECT * FROM Departments;
SELECT * FROM Inventory; 

GO

USE [QueryTraining];
GO



-- OUTPUT with the INSERT statment
--http://stevestedman.com/?p=3726

INSERT INTO dbo.Inventory (Category, ItemSKU, Description, 
						   QuantityOnHand, UnitCost, UnitRetail, Archived)
VALUES
('FOOD', 'MR009-210-ICE', 'MRE - Ice Cream - Chocolate', 1, NULL, 11.95, 0);


SELECT * FROM Inventory; 







INSERT INTO dbo.Inventory (Category, ItemSKU, Description, 
						   QuantityOnHand, UnitCost, UnitRetail, Archived)
OUTPUT INSERTED.*
VALUES
('FOOD', 'MR009-209-ICE', 'MRE - Ice Cream - Rocky Road', 1, NULL, 11.95, 0);


SELECT * FROM Inventory; 




INSERT INTO dbo.Inventory (Category, ItemSKU, Description, 
						   QuantityOnHand, UnitCost, UnitRetail, Archived)
OUTPUT INSERTED.[InventoryID]
VALUES
('FOOD', 'MR009-211-ICE', 'MRE - Ice Cream - Vanilla', 1, NULL, 11.95, 0);



-- What about multiples on the insert statment (SQL 2008 and newer)
--http://stevestedman.com/?p=3726


INSERT INTO dbo.Inventory (Category, ItemSKU, Description, 
						   QuantityOnHand, UnitCost, UnitRetail, Archived)
OUTPUT INSERTED.[InventoryID]
VALUES
('FOOD', 'MR009-212-ICE', 'MRE - Ice Cream - Cookies and Cream', 1, NULL, 11.95, 0),
('FOOD', 'MR009-213-ICE', 'MRE - Ice Cream - Chocolate Chip', 1, NULL, 11.95, 0);










-- OUTPUT with the UPDATE statment
-- http://stevestedman.com/?p=3759

USE [QueryTraining];
GO


SELECT * FROM Inventory; 

UPDATE dbo.Inventory
   SET [Description] = 'MRE - Ice Cream - Double Chocolate Chip'
 WHERE [ItemSKU] = 'MR009-213-ICE';









UPDATE dbo.Inventory
   SET [Description] = 'MRE - Ice Cream - Triple Chocolate Chip'
OUTPUT [DELETED].*, [INSERTED].*
 WHERE [ItemSKU] = 'MR009-213-ICE';




 
SELECT * FROM dbo.Inventory;



-- give me the before and after of the Description update.
UPDATE dbo.Inventory
   SET [Description] = 'MRE - Ice Cream - Strawberry'
OUTPUT [DELETED].[Description], 
       [INSERTED].[Description]
 WHERE [ItemSKU] = 'MR009-213-ICE';
 


-- alias the output 
UPDATE dbo.Inventory
   SET [Description] = 'MRE - Ice Cream - Blueberry'
OUTPUT [DELETED].[Description] AS [Before], 
       [INSERTED].[Description] AS [After]
 WHERE [ItemSKU] = 'MR009-213-ICE';


SELECT * FROM dbo.Inventory;


-- implement a 10% price increase
UPDATE dbo.Inventory
   SET [UnitRetail] = [UnitRetail] * 1.10
OUTPUT [deleted].InventoryID, 
       [deleted].UnitRetail AS PriceBefore, 
	   [inserted].UnitRetail AS PriceAfter;



-- save it to a temp table for further processing....
CREATE TABLE #inventoryTempTable
(
	[InventoryID] [INTEGER],
	[PriceBefore] DECIMAL(10,2),
	[PriceAfter] DECIMAL(10,2)
);

UPDATE dbo.Inventory
   SET [UnitRetail] = [UnitRetail] * 1.10
OUTPUT [deleted].InventoryID, 
       [deleted].UnitRetail AS PriceBefore, 
	   [inserted].UnitRetail AS PriceAfter
  INTO #inventoryTempTable;


SELECT * FROM #inventoryTempTable;
DROP TABLE #inventoryTempTable;










-- OUTPUT with the DELETE statment
USE [QueryTraining];
GO


SELECT * 
  FROM dbo.Inventory;

DELETE dbo.Inventory
OUTPUT [deleted].*
 WHERE [ItemSKU] = 'MR009-213-ICE';


SELECT * 
  FROM dbo.Inventory;


DELETE dbo.Inventory
OUTPUT [deleted].*
 WHERE [ItemSKU] LIKE 'MR%';

SELECT * 
  FROM dbo.Inventory;






-- DELETE and INSERT to another table
GO
CREATE TABLE [dbo].[DeletedInventory](
	[InventoryID] [INTEGER],
	[Category] [VARCHAR](200), 
	[ItemSKU] [VARCHAR](200),
	[Description] [VARCHAR](4000),
	[QuantityOnHand] [INT],
	[UnitCost] DECIMAL(10,2),
	[UnitRetail] DECIMAL(10,2),
	[Archived] BIT
); 
GO

SELECT * 
  FROM dbo.Inventory;

-- delete everything from Inventory and move it to another table.
DELETE dbo.Inventory
OUTPUT [deleted].*
  INTO [dbo].[DeletedInventory];


SELECT * 
  FROM dbo.Inventory;

SELECT * 
  FROM dbo.DeletedInventory;






-- OUTPUT for Auditing
-- http://stevestedman.com/?p=1504

USE [QueryTraining];
GO

SELECT * FROM Departments;

 
CREATE TABLE [DeptAudit] (
 id INT, --would normally be an INT IDENTITY
 old_department VARCHAR (200),
 new_department VARCHAR (200),
 parent INT,
 ModificationDate DATETIME
 );

-- here's the magic to update one table and insert into another in one statement
 
UPDATE dept
 SET dept.department = 'Camp Kitchen'
OUTPUT [deleted].id, [deleted].department,
       [inserted].department, [deleted].parent, GETDATE()
 INTO DeptAudit
 FROM Departments dept
 WHERE dept.id = 8;
 
SELECT * FROM Departments;
SELECT * FROM DeptAudit;


